@@ -65,6 +65,7 @@ let dumb_astcocci_rule_elem = function
  | A.Return _ -> ()
  | A.ReturnExpr (_, ea, _) -> ()
  | A.DefineHeader _ -> ()
+ | A.Undef _ -> ()
  | A.Include _ -> ()
  | A.Default _ -> ()
  | A.Case _ -> ()
@@ -77,6 +78,8 @@ let dumb_astcocci_decl = function
  | A.MacroDecl(fn, _, eas, _, _) -> ()
  | A.Ddots(dots,whencode) -> ()
  | A.MetaDecl _ -> ()
+ | A.MetaField _ -> ()
+ | A.MetaFieldList _ -> ()
  | A.Typedef(d,ty1,ty2,pv) -> ()
  | A.DisjDecl xs -> ()
  | A.OptDecl _ | A.UniqueDecl _ -> ()
@@ -90,6 +93,8 @@ let dumb_astcocci_initialiser = function
   | A.DisjDecl(decls) -> ()
   | A.Ddots(dots,whencode) -> ()
   | A.MetaDecl(name,_,_) -> ()
+  | A.MetaField(name,_,_) -> ()
+  | A.MetaFieldList(name,_,_,_) -> ()
   | A.OptDecl(decl) -> ()
   | A.UniqueDecl(decl) -> ()
 
@@ -142,6 +147,7 @@ let dumb_astcocci_type = function
  | A.FunctionType _ -> ()
  | A.Array (typa, _, eaopt, _) -> ()
  | A.EnumName(en, ena) -> ()
+ | A.EnumDef(ty, lb, ids, rb) -> ()
  | A.StructUnionName(sa, sua) -> ()
  | A.StructUnionDef(ty, lb, decls, rb) -> ()
  | A.TypeName sa -> ()
