@@ -127,6 +127,13 @@ module type PARAM =
 
     val distrf_ini :
       (Ast_cocci.meta_name Ast_cocci.mcode, Ast_c.initialiser) matcher
+    val distrf_inis :
+      (Ast_cocci.meta_name Ast_cocci.mcode,
+       (Ast_c.initialiser, Ast_c.il) Common.either list) matcher
+    val distrf_decl :
+      (Ast_cocci.meta_name Ast_cocci.mcode, Ast_c.declaration) matcher
+    val distrf_field :
+      (Ast_cocci.meta_name Ast_cocci.mcode, Ast_c.field) matcher
 
     val distrf_node :
       (Ast_cocci.meta_name Ast_cocci.mcode, Control_flow_c.node) matcher
@@ -136,6 +143,10 @@ module type PARAM =
       (string Ast_c.wrap, Ast_c.il) Common.either list)
       matcher
 
+    val distrf_enum_fields :
+      (Ast_cocci.meta_name Ast_cocci.mcode,
+       (Ast_c.oneEnumType, Ast_c.il) Common.either list) matcher
+
     val distrf_struct_fields :
       (Ast_cocci.meta_name Ast_cocci.mcode, Ast_c.field list)
       matcher
