@@ -32,7 +32,7 @@ let show_trying = ref false
 
 let track_iso_usage = ref false
 
-type scanner = Glimpse | Grep | Google of string | NoScanner
+type scanner = IdUtils | Glimpse | Grep | Google of string | NoScanner
 let scanner = ref Grep
 
 let pyoutput = ref "coccilib.output.Console"
@@ -60,3 +60,5 @@ let set_defined_virtual_rules s =
   match Str.split (Str.regexp "=") s with
     [name;vl] -> defined_virtual_env := (name,vl) :: !defined_virtual_env
   | _ -> defined_virtual_rules := s :: !defined_virtual_rules
+
+let c_plus_plus = ref false
