@@ -17,9 +17,12 @@ type param_type =
   | ParamList of Ast_c.parameterType Ast_c.wrap2 list
   | Expr of Ast_c.expression
   | ExprList of Ast_c.argument Ast_c.wrap2 list
+  | Decl of Ast_c.declaration
+  | Field of Ast_c.field
+  | FieldList of Ast_c.field list
   | Stmt of Ast_c.statement
 
-let fcts : (string, param_type list -> unit) Hashtbl.t =
+let fcts : (string, param_type list -> string ref list -> unit) Hashtbl.t =
   Hashtbl.create 11 (* Use prime number *)
 
 (* ---------------------------------------------------------------------- *)
