@@ -65,7 +65,7 @@ open Common
  *
  *)
 
-(* forunparser: *)
+(* for unparser: *)
 
 type posl = int * int (* line-col, for MetaPosValList, for position variables *)
  (* with sexp *)
@@ -172,6 +172,7 @@ and fullType = typeQualifier * typeC
  and typeC = typeCbis wrap (* todo reput wrap3 *)
 
   and typeCbis =
+    NoType (* for c++ only *)
   | BaseType        of baseType
 
   | Pointer         of fullType
@@ -186,7 +187,7 @@ and fullType = typeQualifier * typeC
 
   | TypeName   of name * fullType option (* semantic: filled later *)
 
-  | ParenType of fullType (* forunparser: *)
+  | ParenType of fullType (* for unparser: *)
 
   (* gccext: TypeOfType below may seems useless; Why declare a
    *     __typeof__(int) x; ?
@@ -209,6 +210,9 @@ and fullType = typeQualifier * typeC
      and  baseType = Void
                    | IntType   of intType
 		   | FloatType of floatType
+		   | SizeType
+		   | SSizeType
+		   | PtrDiffType
 
 	  (* stdC: type section
            * add  a | SizeT ?
@@ -257,10 +261,10 @@ and fullType = typeQualifier * typeC
 
 
      (* -------------------------------------- *)
-     and enumType = (name * (info (* = *) * constExpression) option)
-                    wrap2 (* , *) list
+     and enumType = oneEnumType wrap2 (* , *) list
                    (* => string * int list *)
 
+     and oneEnumType = name * (info (* = *) * constExpression) option
 
      (* -------------------------------------- *)
      (* return * (params * has "...") *)
@@ -328,12 +332,16 @@ and expression = (expressionbis * exp_info ref (* semantic: *)) wrap3
   | StatementExpr of compound wrap (* ( )     new scope *)
   | Constructor  of fullType * initialiser wrap2 (* , *) list
 
-  (* forunparser: *)
+  (* for unparser: *)
   | ParenExpr of expression
 
+  (* for C++: *)
+  | New of argument
+  | Delete of expression
+
   (* cppext: IfdefExpr TODO *)
 
-  (* cppext: normmally just expression *)
+  (* cppext: normally just expression *)
   and argument = (expression, weird_argument) Common.either
    and weird_argument =
        | ArgType of parameterType
@@ -380,7 +388,6 @@ and expression = (expressionbis * exp_info ref (* semantic: *)) wrap3
 
  and constExpression = expression (* => int *)
 
-
 (* ------------------------------------------------------------------------- *)
 (* C statement *)
 (* ------------------------------------------------------------------------- *)
@@ -510,7 +517,7 @@ and declaration =
   | MacroDecl of (string * argument wrap2 list) wrap (* fakestart *)
 
      and onedecl =
-       { v_namei: (name * (info (* = *) * initialiser) option) option;
+       { v_namei: (name * v_init) option;
          v_type: fullType;
          (* semantic: set in type annotated and used in cocci_vs_c
           * when we transform some initialisation into affectation
@@ -520,6 +527,9 @@ and declaration =
          v_local: local_decl; (* cocci: *)
          v_attr: attribute list; (* gccext: *)
        }
+     and v_init =
+       NoInit | ValInit of info * initialiser
+     | ConstrInit of argument wrap2 (* , *) list wrap
      and storage       = storagebis * bool (* gccext: inline or not *)
      and storagebis    = NoSto | StoTypedef | Sto of storageClass
      and storageClass  = Auto  | Static | Register | Extern
@@ -575,7 +585,6 @@ and definition = definitionbis wrap (* ( ) { } fakestart sto *)
 and cpp_directive =
   | Define of define
   | Include of includ
-  | Undef of string wrap
   | PragmaAndCo of il
 (*| Ifdef ? no, ifdefs are handled differently, cf ifdef_directive below *)
 
@@ -583,6 +592,7 @@ and define = string wrap (* #define s eol *) * (define_kind * define_val)
    and define_kind =
    | DefineVar
    | DefineFunc   of ((string wrap) wrap2 list) wrap (* () *)
+   | Undef
    and define_val =
      (* most common case; e.g. to define int constant *)
      | DefineExpr of expression
@@ -699,6 +709,9 @@ and metavars_binding = (Ast_cocci.meta_name, metavar_binding_kind) assoc
 
   | MetaTypeVal      of fullType
   | MetaInitVal      of initialiser
+  | MetaDeclVal      of declaration
+  | MetaFieldVal     of field
+  | MetaFieldListVal of field list
   | MetaStmtVal      of statement
 
   (* Could also be in Lib_engine.metavars_binding2 with the ParenVal,
