@@ -81,12 +81,13 @@ let is_known_typdef =
  * because it would compute msg_typedef at compile time when
  * the flag debug_typedef is always false
  *)
-let msg_typedef s =
+let msg_typedef s n =
   incr Stat.nTypedefInfer;
   msg_gen (!Flag_parsing_c.debug_typedef)
     is_known_typdef
     (fun s ->
-      pr2_cpp ("TYPEDEF: promoting: " ^ s)
+      (*pr2_cpp (Printf.sprintf "TYPEDEF: promoting(%d): %s" n s)*)
+      pr2_cpp (Printf.sprintf "TYPEDEF: promoting: %s" s)
     )
     s
 
@@ -395,6 +396,7 @@ let rec set_ifdef_parenthize_info xs =
  * [TCommentSpace "\n"; TDefEOL] but if TDefEOL is a fakeinfo then we will
  * not synchronize on it and so we will not print the "\n".
  * A solution would be to put the TDefEOL before the "\n".
+ * (jll: tried to do this, see the comment "Put end of line..." below)
  *
  * todo?: could put a ExpandedTok for that ?
  *)
@@ -418,6 +420,10 @@ let rec define_line_1 acc xs =
       let line = Ast_c.line_of_info ii in
       let acc = (TDefine ii) :: acc in
       define_line_2 acc line ii xs
+  | TUndef ii::xs ->
+      let line = Ast_c.line_of_info ii in
+      let acc = (TUndef ii) :: acc in
+      define_line_2 acc line ii xs
   | TCppEscapedNewline ii::xs ->
       pr2 ("SUSPICIOUS: a \\ character appears outside of a #define at");
       pr2 (Ast_c.strloc_of_info ii);
@@ -447,12 +453,29 @@ and define_line_2 acc line lastinfo xs =
       | x ->
           if line' =|= line
           then define_line_2 (x::acc) line info xs
-          else define_line_1 (mark_end_define lastinfo::acc) (x::xs)
+          else
+	    (* Put end of line token before the newline.  A newline at least
+	       must be there because the line changed and because we saw a
+	       #define previously to get to this function at all *)
+	    define_line_1
+	      ((List.hd acc)::(mark_end_define lastinfo::(List.tl acc)))
+	      (x::xs)
       )
 
 let rec define_ident acc xs =
   match xs with
   | [] -> List.rev acc
+  | TUndef ii::xs ->
+      let acc = TUndef ii :: acc in
+      (match xs with
+	TCommentSpace i1::TIdent (s,i2)::xs ->
+	  let acc = (TCommentSpace i1) :: acc in
+	  let acc = (TIdentDefine (s,i2)) :: acc in
+          define_ident acc xs
+      | _ ->
+          pr2 "WEIRD: weird #define body";
+          define_ident acc xs
+      )
   | TDefine ii::xs ->
       let acc = TDefine ii :: acc in
       (match xs with
@@ -1489,8 +1512,8 @@ let insert_virtual_positions l =
 	| _ -> x::skip_fake xs in
   skip_fake l
 
-
 (* ------------------------------------------------------------------------- *)
+
 let fix_tokens_cpp2 ~macro_defs tokens =
   let tokens2 = ref (tokens +> Common.acc_map TV.mk_token_extended) in
 
@@ -1612,11 +1635,25 @@ let not_struct_enum = function
   | (Parser_c.Tstruct _ | Parser_c.Tunion _ | Parser_c.Tenum _)::_ -> false
   | _ -> true
 
+let pointer = function
+    TMul _ -> true
+  | TAnd _ when !Flag.c_plus_plus -> true
+  | _ -> false
 
 let lookahead2 ~pass next before =
 
   match (next, before) with
 
+  (* c++ hacks *)
+  (* yy xx(   and in function *)
+  | TOPar i1::_,              TIdent(s,i2)::TypedefIdent _::_
+      when !Flag.c_plus_plus && (LP.current_context () = (LP.InFunction)) ->
+        pr2_cpp("constructed_object: "  ^s);
+        TOParCplusplusInit i1
+  | TypedefIdent(s,i)::TOPar i1::_,_
+      when !Flag.c_plus_plus && (LP.current_context () = (LP.InFunction)) ->
+	TIdent(s,i)
+
   (*-------------------------------------------------------------*)
   (* typedef inference, parse_typedef_fix3 *)
   (*-------------------------------------------------------------*)
@@ -1637,7 +1674,7 @@ let lookahead2 ~pass next before =
 
       LP.disable_typedef();
 
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 1; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (* xx yy *)
@@ -1646,7 +1683,7 @@ let lookahead2 ~pass next before =
         ->
          (* && not_annot s2 BUT lead to false positive*)
 
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 2; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
@@ -1654,7 +1691,7 @@ let lookahead2 ~pass next before =
   | (TIdent (s, i1)::Tinline i2::_  , _) when not_struct_enum before
       && ok_typedef s
       ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 3; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
@@ -1663,17 +1700,17 @@ let lookahead2 ~pass next before =
     when not_struct_enum before && (LP.current_context() =*= LP.InParameter)
       && ok_typedef s
       ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 4; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (* xx* [,)] *)
   (* specialcase:  [,(] xx* [,)] *)
-  | (TIdent (s, i1)::TMul _::(TComma _|TCPar _)::_ , (*(TComma _|TOPar _)::*)_ )
-    when not_struct_enum before
+  | (TIdent (s, i1)::ptr::(TComma _|TCPar _)::_ , (*(TComma _|TOPar _)::*)_ )
+    when pointer ptr && not_struct_enum before
         (* && !LP._lexer_hint = Some LP.ParameterDeclaration *)
       && ok_typedef s
     ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 5; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
@@ -1684,7 +1721,7 @@ let lookahead2 ~pass next before =
       (* && !LP._lexer_hint = Some LP.ParameterDeclaration *)
       && ok_typedef s
     ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 6; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
@@ -1696,7 +1733,7 @@ let lookahead2 ~pass next before =
       && ok_typedef s
       ->
 
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 7; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (* xx const *)
@@ -1706,25 +1743,25 @@ let lookahead2 ~pass next before =
       (* && !LP._lexer_hint = Some LP.ParameterDeclaration *)
       ->
 
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 8; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
   (* xx * const *)
-  | (TIdent (s, i1)::TMul _::(Tconst _ | Tvolatile _|Trestrict _)::_ , _ )
-      when not_struct_enum before
+  | (TIdent (s, i1)::ptr::(Tconst _ | Tvolatile _|Trestrict _)::_ , _ )
+      when pointer ptr && not_struct_enum before
       && ok_typedef s
       ->
       (* && !LP._lexer_hint = Some LP.ParameterDeclaration *)
 
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 9; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
   (* ( const xx)  *)
   | (TIdent (s, i1)::TCPar _::_,  (Tconst _ | Tvolatile _|Trestrict _)::TOPar _::_) when
       ok_typedef s ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 10; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
@@ -1734,7 +1771,7 @@ let lookahead2 ~pass next before =
     when not_struct_enum before
       && ok_typedef s
     ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 11; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (* [(,] xx [   AND parameterdeclaration *)
@@ -1742,7 +1779,7 @@ let lookahead2 ~pass next before =
       when (LP.current_context() =*= LP.InParameter)
       && ok_typedef s
      ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 12; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (*------------------------------------------------------------*)
@@ -1751,61 +1788,62 @@ let lookahead2 ~pass next before =
   (*------------------------------------------------------------*)
 
   (* static xx * yy  *)
-  | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::_ ,
+  | (TIdent (s, i1)::ptr::TIdent (s2, i2)::_ ,
      (Tregister _|Tstatic _  |Tvolatile _|Tconst _|Trestrict _)::_) when
-      ok_typedef s
+      pointer ptr && ok_typedef s
         ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 13; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (*  TODO  xx * yy ; AND in start of compound element  *)
 
 
   (*  xx * yy,      AND  in paramdecl *)
-  | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::TComma _::_ , _)
+  | (TIdent (s, i1)::ptr::TIdent (s2, i2)::TComma _::_ , _)
     when not_struct_enum before && (LP.current_context() =*= LP.InParameter)
-      && ok_typedef s
+      && pointer ptr && ok_typedef s
       ->
 
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 14; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
   (*  xx * yy ;     AND in Toplevel, except when have = before  *)
   | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::TPtVirg _::_ , TEq _::_) ->
       TIdent (s, i1)
-  | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::TPtVirg _::_ , _)
-    when not_struct_enum before && (LP.is_top_or_struct (LP.current_context ()))
+  | (TIdent (s, i1)::ptr::TIdent (s2, i2)::TPtVirg _::_ , _)
+    when not_struct_enum before && pointer ptr &&
+      (LP.is_top_or_struct (LP.current_context ()))
       ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 15; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (*  xx * yy ,     AND in Toplevel  *)
-  | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::TComma _::_ , _)
+  | (TIdent (s, i1)::ptr::TIdent (s2, i2)::TComma _::_ , _)
     when not_struct_enum before && (LP.current_context () =*= LP.InTopLevel)
-      && ok_typedef s
+      && ok_typedef s && pointer ptr
       ->
 
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 16; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (*  xx * yy (     AND in Toplevel  *)
-  | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::TOPar _::_ , _)
+  | (TIdent (s, i1)::ptr::TIdent (s2, i2)::TOPar _::_ , _)
     when not_struct_enum before
       && (LP.is_top_or_struct (LP.current_context ()))
-      && ok_typedef s
+      && ok_typedef s && pointer ptr
       ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 17; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (* xx * yy [ *)
   (* todo? enough ? cos in struct def we can have some expression ! *)
-  | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::TOCro _::_ , _)
+  | (TIdent (s, i1)::ptr::TIdent (s2, i2)::TOCro _::_ , _)
     when not_struct_enum before &&
       (LP.is_top_or_struct (LP.current_context ()))
-      && ok_typedef s
+      && ok_typedef s && pointer ptr
       ->
-      msg_typedef s;  LP.add_typedef_root s;
+      msg_typedef s 18;  LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (* u16: 10; in struct *)
@@ -1813,7 +1851,7 @@ let lookahead2 ~pass next before =
     when       (LP.is_top_or_struct (LP.current_context ()))
       && ok_typedef s
       ->
-      msg_typedef s;  LP.add_typedef_root s;
+      msg_typedef s 19;  LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
@@ -1826,54 +1864,54 @@ let lookahead2 ~pass next before =
       (take_safe 1 !passed_tok <> [Tenum]))
       &&
       !LP._lexer_hint = Some LP.Toplevel ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 20; LP.add_typedef_root s;
       TypedefIdent s
      *)
 
   (*  xx * yy =  *)
-  | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::TEq _::_ , _)
+  | (TIdent (s, i1)::ptr::TIdent (s2, i2)::TEq _::_ , _)
     when not_struct_enum before
-      && ok_typedef s
+      && ok_typedef s && pointer ptr
       ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 21; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
   (*  xx * yy)      AND in paramdecl *)
-  | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::TCPar _::_ , _)
+  | (TIdent (s, i1)::ptr::TIdent (s2, i2)::TCPar _::_ , _)
       when not_struct_enum before && (LP.current_context () =*= LP.InParameter)
-      && ok_typedef s
+      && ok_typedef s && pointer ptr
         ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 22; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
   (*  xx * yy; *) (* wrong ? *)
-  | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::TPtVirg _::_ ,
+  | (TIdent (s, i1)::ptr::TIdent (s2, i2)::TPtVirg _::_ ,
      (TOBrace _| TPtVirg _)::_)  when not_struct_enum before
-      && ok_typedef s
+      && ok_typedef s & pointer ptr
         ->
-      msg_typedef s;  LP.add_typedef_root s;
+      msg_typedef s 23;  LP.add_typedef_root s;
       msg_maybe_dangereous_typedef s;
       TypedefIdent (s, i1)
 
 
   (*  xx * yy,  and ';' before xx *) (* wrong ? *)
-  | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::TComma _::_ ,
+  | (TIdent (s, i1)::ptr::TIdent (s2, i2)::TComma _::_ ,
      (TOBrace _| TPtVirg _)::_) when
-      ok_typedef s
+      ok_typedef s && pointer ptr
     ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 24; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
   (* xx_t * yy *)
-  | (TIdent (s, i1)::TMul _::TIdent (s2, i2)::_ , _)
+  | (TIdent (s, i1)::ptr::TIdent (s2, i2)::_ , _)
       when s ==~ regexp_typedef && not_struct_enum before
         (* struct user_info_t sometimes *)
-      && ok_typedef s
+      && ok_typedef s && pointer ptr
         ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 25; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (*  xx ** yy *)  (* wrong ? *)
@@ -1882,7 +1920,7 @@ let lookahead2 ~pass next before =
         (* && !LP._lexer_hint = Some LP.ParameterDeclaration *)
       && ok_typedef s
       ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 26; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (* xx *** yy *)
@@ -1891,7 +1929,7 @@ let lookahead2 ~pass next before =
       && ok_typedef s
         (* && !LP._lexer_hint = Some LP.ParameterDeclaration *)
       ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 27; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (*  xx ** ) *)
@@ -1900,7 +1938,7 @@ let lookahead2 ~pass next before =
         (* && !LP._lexer_hint = Some LP.ParameterDeclaration *)
       && ok_typedef s
       ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 28; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
@@ -1920,7 +1958,7 @@ let lookahead2 ~pass next before =
       && ok_typedef s
       ->
 
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 29; LP.add_typedef_root s;
       (*TOPar info*)
       TypedefIdent (s, i1)
 
@@ -1933,7 +1971,7 @@ let lookahead2 ~pass next before =
     when not (TH.is_stuff_taking_parenthized x)
       && ok_typedef s
         ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 30; LP.add_typedef_root s;
       (* TOPar info *)
       TypedefIdent (s, i1)
   *)
@@ -1942,16 +1980,16 @@ let lookahead2 ~pass next before =
     (TOPar info)::(TEq _ |TEqEq _)::_)
     when ok_typedef s
         ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 31; LP.add_typedef_root s;
       (* TOPar info *)
       TypedefIdent (s, i1)
 
 
   (*  (xx * ) yy *)
-  | (TIdent (s, i1)::TMul _::TCPar _::TIdent (s2, i2)::_ , (TOPar info)::_) when
-      ok_typedef s
+  | (TIdent (s, i1)::ptr::TCPar _::TIdent (s2, i2)::_ , (TOPar info)::_)
+    when ok_typedef s && pointer ptr
         ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 32; LP.add_typedef_root s;
       (*TOPar info*)
       TypedefIdent (s,i1)
 
@@ -1961,7 +1999,7 @@ let lookahead2 ~pass next before =
       when (*s ==~ regexp_typedef && *) not (TH.is_stuff_taking_parenthized x)
       && ok_typedef s
         ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 33; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
@@ -1978,7 +2016,7 @@ let lookahead2 ~pass next before =
       when not_struct_enum before
       && ok_typedef s
         ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 34; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
   (* x* ( *y )(params),  function pointer 2 *)
@@ -1986,7 +2024,7 @@ let lookahead2 ~pass next before =
       when not_struct_enum before
       && ok_typedef s
         ->
-      msg_typedef s; LP.add_typedef_root s;
+      msg_typedef s 35; LP.add_typedef_root s;
       TypedefIdent (s, i1)
 
 
@@ -2020,7 +2058,7 @@ let lookahead2 ~pass next before =
       end
       else x
 
-  | (TUndef (id, ii) as x)::_, _
+  | (TUndef (ii) as x)::_, _
       ->
         if (pass >= 2)
         then begin
@@ -2050,7 +2088,6 @@ let lookahead2 ~pass next before =
        *)
         ->
 
-
       if s ==~ regexp_foreach &&
         is_really_foreach (Common.take_safe forLOOKAHEAD rest)
 
