@@ -158,7 +158,7 @@ let is_statement = function
 
 let is_start_of_something = function
   | Tchar _  | Tshort _ | Tint _ | Tdouble _ |  Tfloat _ | Tlong _
-  | Tunsigned _ | Tsigned _ | Tvoid _
+  | Tunsigned _ | Tsigned _ | Tvoid _ | Tsize_t _ | Tssize_t _ | Tptrdiff_t _
   | Tauto _ | Tregister _ | Textern _ | Tstatic _
   | Tconst _ | Tvolatile _
   | Ttypedef _
@@ -222,6 +222,7 @@ let info_of_tok = function
   | TAssign  (assignOp, i) -> i
 
   | TIdent  (s, i) -> i
+  | Tconstructorname  (s, i) -> i
   | TypedefIdent  (s, i) -> i
 
   | TInt  (s, i) -> i
@@ -229,7 +230,7 @@ let info_of_tok = function
   | TDefine (ii) -> ii
   | TInclude (includes, filename, inifdef, i1) ->     i1
 
-  | TUndef (s, ii) -> ii
+  | TUndef (ii) -> ii
   | TCppDirectiveOther (ii) -> ii
 
   | TIncludeStart (i1, inifdef) ->     i1
@@ -323,6 +324,9 @@ let info_of_tok = function
   | Tunsigned            (i) -> i
   | Tsigned              (i) -> i
   | Tvoid                (i) -> i
+  | Tsize_t              (i) -> i
+  | Tssize_t             (i) -> i
+  | Tptrdiff_t           (i) -> i
   | Tauto                (i) -> i
   | Tregister            (i) -> i
   | Textern              (i) -> i
@@ -354,6 +358,9 @@ let info_of_tok = function
   | TattributeNoarg           (i) -> i
   | Tinline              (i) -> i
   | Ttypeof              (i) -> i
+  | Tnew                 (i) -> i
+  | Tdelete              (i) -> i
+  | TOParCplusplusInit   (i) -> i
 
   | EOF                  (i) -> i
 
@@ -367,13 +374,14 @@ let visitor_info_of_tok f = function
   | TFloat ((s, floatType), i) -> TFloat ((s, floatType), f i)
   | TAssign  (assignOp, i)     -> TAssign  (assignOp, f i)
 
-  | TIdent  (s, i)       -> TIdent  (s, f i)
-  | TypedefIdent  (s, i) -> TypedefIdent  (s, f i)
-  | TInt  (s, i)         -> TInt  (s, f i)
+  | TIdent  (s, i)         -> TIdent  (s, f i)
+  | Tconstructorname(s, i) -> Tconstructorname  (s, f i)
+  | TypedefIdent  (s, i)   -> TypedefIdent  (s, f i)
+  | TInt  (s, i)           -> TInt  (s, f i)
 
   | TDefine (i1) -> TDefine(f i1)
 
-  | TUndef (s,i1) -> TUndef(s, f i1)
+  | TUndef (i1) -> TUndef(f i1)
   | TCppDirectiveOther (i1) -> TCppDirectiveOther(f i1)
 
   | TInclude (includes, filename, inifdef, i1) ->
@@ -473,6 +481,9 @@ let visitor_info_of_tok f = function
   | Tunsigned            (i) -> Tunsigned            (f i)
   | Tsigned              (i) -> Tsigned              (f i)
   | Tvoid                (i) -> Tvoid                (f i)
+  | Tsize_t              (i) -> Tsize_t              (f i)
+  | Tssize_t             (i) -> Tssize_t             (f i)
+  | Tptrdiff_t           (i) -> Tptrdiff_t           (f i)
   | Tauto                (i) -> Tauto                (f i)
   | Tregister            (i) -> Tregister            (f i)
   | Textern              (i) -> Textern              (f i)
@@ -501,9 +512,12 @@ let visitor_info_of_tok f = function
   | Tsizeof              (i) -> Tsizeof              (f i)
   | Tasm                 (i) -> Tasm                 (f i)
   | Tattribute           (i) -> Tattribute           (f i)
-  | TattributeNoarg           (i) -> TattributeNoarg           (f i)
+  | TattributeNoarg      (i) -> TattributeNoarg      (f i)
   | Tinline              (i) -> Tinline              (f i)
   | Ttypeof              (i) -> Ttypeof              (f i)
+  | Tnew                 (i) -> Tnew                 (f i)
+  | Tdelete              (i) -> Tdelete              (f i)
+  | TOParCplusplusInit   (i) -> TOParCplusplusInit   (f i)
   | EOF                  (i) -> EOF                  (f i)
 
 
