@@ -336,6 +336,7 @@ let rec find_final_type ty env =
 let rec type_unfold_one_step ty env =
 
   match Ast_c.unwrap_typeC ty with
+  | NoType        -> ty
   | BaseType x    -> ty
   | Pointer t     -> ty
   | Array (e, t)  -> ty
@@ -391,6 +392,8 @@ let rec type_unfold_one_step ty env =
  *)
 let rec typedef_fix ty env =
   match Ast_c.unwrap_typeC ty with
+  | NoType  ->
+      ty
   | BaseType x  ->
       ty
   | Pointer t ->
@@ -611,8 +614,8 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
 
     (* -------------------------------------------------- *)
     (* todo: should analyse the 's' for int to know if unsigned or not *)
-    | Constant (String (s,kind)) -> make_info_def (type_of_s "char *")
-    | Constant MultiString _  -> make_info_def (type_of_s "char *")
+    | Constant (String (s,kind)) -> make_info_def (type_of_s "char []")
+    | Constant MultiString _  -> make_info_def (type_of_s "char []")
     | Constant (Char   (s,kind)) -> make_info_def (type_of_s "char")
     | Constant (Int (s,kind)) ->
 	(* this seems really unpleasant, but perhaps the type needs to be set
@@ -647,7 +650,6 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
     | FunCall (e1, args) ->
      (match Ast_c.unwrap_expr e1 with
      | Ident (ident) ->
-
         (* recurse *)
         args +> List.iter (fun (e,ii) ->
           (* could typecheck if arguments agree with prototype *)
@@ -701,8 +703,11 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
                 | DefineVar, _ ->
                     pr2 ("Type_annoter: not a macro-func: " ^ s);
                     Type_c.noTypeHere
+                | Undef, _ ->
+                    pr2 ("Type_annoter: not a macro-func: " ^ s);
+                    Type_c.noTypeHere
                 | DefineFunc _, _ ->
-                    (* normally the FunCall case should have catch it *)
+                    (* normally the FunCall case should have caught it *)
                     pr2 ("Type_annoter: not a macro-func-expr: " ^ s);
                     Type_c.noTypeHere
                 )
@@ -750,6 +755,9 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
                     (* normally the FunCall case should have catch it *)
                     pr2 ("Type_annoter: not a macro-var: " ^ s);
                     Type_c.noTypeHere
+                | Undef, _ ->
+                    pr2 ("Type_annoter: not a expression: " ^ s);
+                    Type_c.noTypeHere
                 )
             | None ->
                 (match lookup_opt_env lookup_enum s with
@@ -860,8 +868,12 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
     (* todo? lub, hmm maybe not, cos type must be e1 *)
     | Assignment (e1, op, e2) ->
         k expr;
-        (* value of an assignment is the value of the RHS expression *)
-        Ast_c.get_type_expr e2
+        (* value of an assignment is the value of the RHS expression, but its
+           type is the type of the lhs expression.  Use the rhs exp if no
+	   information is available *)
+        (match Ast_c.get_type_expr e1 with
+	  (None,_) -> Ast_c.get_type_expr e2
+	| (Some ty,t) -> (Some ty,t))
     | Sequence (e1, e2) ->
         k expr;
         Ast_c.get_type_expr e2
@@ -899,7 +911,7 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
 
     | SizeOfType _|SizeOfExpr _ ->
         k expr; (* recurse to set the types-ref of sub expressions *)
-        make_info_def (type_of_s "int")
+        make_info_def (type_of_s "size_t")
 
     | Constructor (ft, ini) ->
         k expr; (* recurse to set the types-ref of sub expressions *)
@@ -928,6 +940,16 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
             | _ -> k expr; Type_c.noTypeHere
           *)
 
+    | New ty ->
+	k expr;
+	pr2_once "Type annotater:not handling New";
+	Type_c.noTypeHere (* TODO *)
+
+    | Delete e ->
+	k expr;
+	pr2_once "Type annotater:not handling Delete";
+	Type_c.noTypeHere (* TODO *)
+
   in
   Ast_c.set_type_expr expr ty
 
@@ -1023,7 +1045,6 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
 
           add_binding (Macro (s, (defkind, defval) )) true;
 
-      | Undef _
       | PragmaAndCo _ -> ()
     );
 
@@ -1069,9 +1090,13 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
 
                   if need_annotate_body then begin
                     (* int x = sizeof(x) is legal so need process ini *)
-                    iniopt +> Common.do_option (fun (info, ini) ->
-                      Visitor_c.vk_ini bigf ini
-                    );
+		    match iniopt with
+		      Ast_c.NoInit -> ()
+		    | Ast_c.ValInit(iini,init) -> Visitor_c.vk_ini bigf init
+		    | Ast_c.ConstrInit((args,_)) ->
+			args +> List.iter (fun (e,ii) ->
+			  Visitor_c.vk_argument bigf e
+			)
                   end
             );
           );
@@ -1254,6 +1279,14 @@ let annotate_test_expressions prog =
     | Binary(e1,Logical(OrLog),e2) -> propagate_test e1; propagate_test e2
     | Unary(e1,Not) -> propagate_test e1
     | ParenExpr(e) -> propagate_test e
+    | FunCall(e,args) -> (* not very nice, but so painful otherwise *)
+	(match (unwrap e,args) with
+	  ((Ident(i),_),[(Left a,_)]) ->
+	    let nm = str_of_name i in
+	    if List.mem nm ["likely";"unlikely"]
+	    then propagate_test a
+		else ()
+	| _ -> ())
     | _ -> () in
 
   let bigf = { Visitor_c.default_visitor_c with
