@@ -35,6 +35,9 @@ exception CantBeInPlus
 type pos = Before | After | InPlace
 type nlhint = StartBox | EndBox | SpaceOrNewline of string ref
 
+let get_string_info = function
+    Ast.Noindent s | Ast.Indent s | Ast.Space s -> s
+
 let unknown = -1
 
 let rec do_all
@@ -110,8 +113,7 @@ let print_around printer term = function
       print_anything bef; printer term; print_anything aft in
 
 let print_string_befaft fn fn1 x info =
-  let print ln col =
-    function Ast.Noindent s | Ast.Indent s -> print_string s ln col in
+  let print ln col s = print_string (get_string_info s) ln col in
   List.iter
     (function (s,ln,col) -> fn1(); print ln col s; force_newline())
     info.Ast.strbef;
@@ -145,11 +147,11 @@ let mcode fn (s,info,mc,pos) =
 		  let str =
 		    match str with
 		      Ast.Noindent s -> unindent false; s
-		    | Ast.Indent s -> s in
+		    | Ast.Indent s -> s
+		    | Ast.Space s -> s in
 		  print_string str line col; Some line
 	      |	Some lb when line =|= lb ->
-		  let str = match str with Ast.Noindent s | Ast.Indent s -> s in
-		  print_string str line col; Some line
+		  print_string (get_string_info str) line col; Some line
 	      |	_ ->
 		  force_newline();
 		  (* not super elegant to put side-effecting unindent in a let
@@ -157,7 +159,8 @@ let mcode fn (s,info,mc,pos) =
 		  let str =
 		    match str with
 		      Ast.Noindent s -> unindent false; s
-		    | Ast.Indent s -> s in
+		    | Ast.Indent s -> s
+		    | Ast.Space s -> s in
 		  print_string str line col; Some line)
 	  lb comments in
       let line_before = print_comments None info.Ast.strbef in
@@ -238,6 +241,11 @@ let nest_dots starter ender fn f d =
   mcode print_string ender
 in
 
+let print_disj_list fn l =
+  print_text "\n(\n";
+  print_between (function _ -> print_text "\n|\n") fn l;
+  print_text "\n)\n" in
+
 (* --------------------------------------------------------------------- *)
 (* Identifier *)
 
@@ -260,6 +268,10 @@ let rec ident i =
 			       | _ -> raise Impossible
 			    )
 
+    | Ast.DisjId(id_list) ->
+	if generating
+	then print_disj_list ident id_list
+	else raise CantBeInPlus
     | Ast.OptIdent(_) | Ast.UniqueIdent(_) ->
 	raise CantBeInPlus
 
@@ -268,11 +280,6 @@ in
 (* --------------------------------------------------------------------- *)
 (* Expression *)
 
-let print_disj_list fn l =
-  print_text "\n(\n";
-  print_between (function _ -> print_text "\n|\n") fn l;
-  print_text "\n)\n" in
-
 let rec expression e =
   match Ast.unwrap e with
     Ast.Ident(id) -> ident id
@@ -460,8 +467,14 @@ and typeC ty =
   | Ast.Array(ty,lb,size,rb) ->
       fullType ty; mcode print_string lb; print_option expression size;
       mcode print_string rb
-  | Ast.EnumName(kind,name) -> mcode print_string kind; pr_space();
-      ident name
+  | Ast.EnumName(kind,name) ->
+      mcode print_string kind;
+      print_option_prespace ident name
+  | Ast.EnumDef(ty,lb,ids,rb) ->
+      fullType ty; ft_space ty;
+      mcode print_string lb;
+      dots force_newline expression ids;
+      mcode print_string rb
   | Ast.StructUnionName(kind,name) ->
       mcode structUnion kind; print_option_prespace ident name
   | Ast.StructUnionDef(ty,lb,decls,rb) ->
@@ -485,6 +498,9 @@ and baseType = function
   | Ast.FloatType -> print_string "float"
   | Ast.LongType -> print_string "long"
   | Ast.LongLongType -> print_string "long long"
+  | Ast.SizeType -> print_string "size_t "
+  | Ast.SSizeType -> print_string "ssize_t "
+  | Ast.PtrDiffType -> print_string "ptrdiff_t "
 
 and structUnion = function
     Ast.Struct -> print_string "struct"
@@ -564,7 +580,27 @@ and ft_space ty =
 
 and declaration d =
   match Ast.unwrap d with
-    Ast.Init(stg,ty,id,eq,ini,sem) ->
+    Ast.MetaDecl(name,_,_) ->
+      handle_metavar name
+	(function
+	    Ast_c.MetaDeclVal d ->
+              pretty_print_c.Pretty_print_c.decl d
+          | _ -> raise Impossible)
+  | Ast.MetaField(name,_,_) ->
+      handle_metavar name
+	(function
+	    Ast_c.MetaFieldVal f ->
+              pretty_print_c.Pretty_print_c.field f
+          | _ -> raise Impossible)
+
+  | Ast.MetaFieldList(name,_,_,_) ->
+      handle_metavar name
+	(function
+	    Ast_c.MetaFieldListVal f ->
+	      print_between force_newline pretty_print_c.Pretty_print_c.field f
+          | _ -> raise Impossible)
+
+  | Ast.Init(stg,ty,id,eq,ini,sem) ->
       print_option (mcode storage) stg;
       print_option (function _ -> pr_space()) stg;
       print_named_type ty id;
@@ -584,7 +620,7 @@ and declaration d =
       mcode print_string stg;
       fullType ty; typeC id;
       mcode print_string sem
-  | Ast.DisjDecl(_) | Ast.MetaDecl(_,_,_) -> raise CantBeInPlus
+  | Ast.DisjDecl(_) -> raise CantBeInPlus
   | Ast.Ddots(_,_) -> raise CantBeInPlus
   | Ast.OptDecl(decl)  | Ast.UniqueDecl(decl) ->
       raise CantBeInPlus
@@ -600,7 +636,16 @@ and initialiser nlcomma i =
             pretty_print_c.Pretty_print_c.init ini
         | _ -> raise Impossible)
   | Ast.InitExpr(exp) -> expression exp
-  | Ast.InitList(_,lb,initlist,rb,[]) ->
+  | Ast.ArInitList(lb,initlist,rb) ->
+      (match Ast.undots initlist with
+	[] -> mcode print_string lb; mcode print_string rb
+      |	_ ->
+	  mcode print_string lb; start_block();
+	  dots force_newline (initialiser false) initlist;
+	  end_block(); mcode print_string rb)
+  | Ast.StrInitList(_,lb,[],rb,[]) ->
+      mcode print_string lb; mcode print_string rb
+  | Ast.StrInitList(_,lb,initlist,rb,[]) ->
       mcode print_string lb; start_block();
       (* awkward, because the comma is separate from the initialiser *)
       let rec loop = function
@@ -609,7 +654,8 @@ and initialiser nlcomma i =
 	| x::xs -> initialiser nlcomma x; loop xs in
       loop initlist;
       end_block(); mcode print_string rb
-  | Ast.InitList(_,lb,initlist,rb,_) -> failwith "unexpected whencode in plus"
+  | Ast.StrInitList(_,lb,initlist,rb,_) ->
+      failwith "unexpected whencode in plus"
   | Ast.InitGccExt(designators,eq,ini) ->
       List.iter designator designators; pr_space();
       mcode print_string eq; pr_space(); initialiser nlcomma ini
@@ -618,6 +664,17 @@ and initialiser nlcomma i =
   | Ast.IComma(comma) ->
       mcode print_string comma;
       if nlcomma then force_newline()
+  | Ast.Idots(dots,Some whencode) ->
+      if generating
+      then
+	(mcode print_string dots;
+	 print_text "   when != ";
+	 initialiser nlcomma whencode)
+      else raise CantBeInPlus
+  | Ast.Idots(dots,None) ->
+      if generating
+      then mcode print_string dots
+      else raise CantBeInPlus
   | Ast.OptIni(ini) | Ast.UniqueIni(ini) ->
       raise CantBeInPlus
 
@@ -645,7 +702,11 @@ and parameterTypeDef p =
               pretty_print_c.Pretty_print_c.param p
           | _ -> raise Impossible)
   | Ast.MetaParamList(name,_,_,_) ->
-      failwith "not handling MetaParamList"
+      handle_metavar name
+	(function
+	    Ast_c.MetaParamListVal p ->
+              pretty_print_c.Pretty_print_c.paramlist p
+          | _ -> raise Impossible)
 
   | Ast.PComma(cm) -> mcode print_string cm
   | Ast.Pdots(dots) | Ast.Pcircles(dots) when generating ->
@@ -752,6 +813,8 @@ and rule_elem arity re =
   | Ast.TopInit(init) -> initialiser false init
   | Ast.Include(inc,s) ->
       mcode print_string inc; print_text " "; mcode inc_file s
+  | Ast.Undef(def,id) ->
+      mcode print_string def; pr_space(); ident id
   | Ast.DefineHeader(def,id,params) ->
       mcode print_string def; pr_space(); ident id;
       print_define_parameters params
@@ -966,8 +1029,16 @@ let rec pp_any = function
 
   | Ast.ConstVolTag(x) -> const_vol x unknown unknown; false
   | Ast.Pragma(xs) ->
-      let print = function Ast.Noindent s | Ast.Indent s -> print_text s in
-      print_between force_newline print xs; false
+      (match xs with (Ast.Space s)::_ -> pr_space() | _ -> ());
+      let rec loop = function
+	  [] -> ()
+	| [(Ast.Indent s | Ast.Noindent s)] -> print_text s
+	| (Ast.Space s) :: (((Ast.Indent _ | Ast.Noindent _) :: _) as rest) ->
+	    print_text s; force_newline(); loop rest
+	| (Ast.Space s) :: rest -> print_text s; pr_space(); loop rest
+	| (Ast.Indent s | Ast.Noindent s) :: rest ->
+	    print_text s; force_newline(); loop rest in
+      loop xs; false
   | Ast.Token(x,None) -> print_text x; if_open_brace x
   | Ast.Token(x,Some info) ->
       mcode
@@ -1016,7 +1087,10 @@ in
 	then
 	  let hd = List.hd xxs in
 	  match hd with
-            (Ast.StatementTag s::_) when isfn s ->
+	    (Ast.Pragma l::_)
+	      when List.for_all (function Ast.Space x -> true | _ -> false) l ->
+		()
+          | (Ast.StatementTag s::_) when isfn s ->
 	      force_newline(); force_newline()
 	  | (Ast.Pragma _::_)
           | (Ast.Rule_elemTag _::_) | (Ast.StatementTag _::_)
