@@ -55,7 +55,16 @@ let update_after pragmas (info,x) =
 
 let rec right_decl d =
   match Ast0.unwrap d with
-    Ast0.Init(Some stg,ty,id,eq,ini,sem) ->
+    Ast0.MetaDecl(name,pure) ->
+      call_right right_mcode name d
+	(function name -> Ast0.MetaDecl(name,pure))
+  | Ast0.MetaField(name,pure) ->
+      call_right right_mcode name d
+	(function name -> Ast0.MetaField(name,pure))
+  | Ast0.MetaFieldList(name,lenname,pure) ->
+      call_right right_mcode name d
+	(function name -> Ast0.MetaFieldList(name,lenname,pure))
+  | Ast0.Init(Some stg,ty,id,eq,ini,sem) ->
       call_right right_mcode sem d
 	(function sem -> Ast0.Init(Some stg,ty,id,eq,ini,sem))
   | Ast0.Init(None,ty,id,eq,ini,sem) ->
@@ -143,6 +152,10 @@ let rec right_statement s =
   | Ast0.Include(inc,name) ->
       call_right right_mcode name s
 	(function name -> Ast0.Include(inc,name))
+  | Ast0.Undef(def,id) ->
+      (* nothing available for ident, and not sure code can appear
+	 here anyway *)
+      None
   | Ast0.Define(def,id,params,body) ->
       call_right right_statement_dots body s
 	(function body -> Ast0.Define(def,id,params,body))
@@ -185,6 +198,9 @@ let rec left_ty t =
       call_right left_ty ty t (function ty -> Ast0.Array(ty,lb,size,rb))
   | Ast0.EnumName(kind,name) ->
       call_right left_mcode kind t (function kind -> Ast0.EnumName(kind,name))
+  | Ast0.EnumDef(ty,lb,ids,rb) ->
+      call_right left_ty ty t
+	(function ty -> Ast0.EnumDef(ty,lb,ids,rb))
   | Ast0.StructUnionName(kind,name) ->
       call_right left_mcode kind t
 	(function kind -> Ast0.StructUnionName(kind,name))
@@ -203,22 +219,20 @@ let rec left_ty t =
 
 let rec left_ident i =
   match Ast0.unwrap i with
-      Ast0.Id(name) ->
-	call_right left_mcode name i
-	  (function name -> Ast0.Id(name))
-    | Ast0.MetaId(name,a,b) ->
-	call_right left_mcode name i
-	  (function name -> Ast0.MetaId(name,a,b))
-    | Ast0.MetaFunc(name,a,b) ->
-	call_right left_mcode name i
-	  (function name -> Ast0.MetaFunc(name,a,b))
-    | Ast0.MetaLocalFunc(name,a,b) ->
-	call_right left_mcode name i
-	  (function name -> Ast0.MetaLocalFunc(name,a,b))
-    | Ast0.OptIdent(id) ->
-	call_right left_ident id i (function id -> Ast0.OptIdent(id))
-    | Ast0.UniqueIdent(id) ->
-	call_right left_ident id i (function id -> Ast0.UniqueIdent(id))
+    Ast0.Id(name) ->
+      call_right left_mcode name i (function name -> Ast0.Id(name))
+  | Ast0.MetaId(name,a,b) ->
+      call_right left_mcode name i (function name -> Ast0.MetaId(name,a,b))
+  | Ast0.MetaFunc(name,a,b) ->
+      call_right left_mcode name i (function name -> Ast0.MetaFunc(name,a,b))
+  | Ast0.MetaLocalFunc(name,a,b) ->
+      call_right left_mcode name i
+	(function name -> Ast0.MetaLocalFunc(name,a,b))
+  | Ast0.DisjId(starter,ids,mids,ender) -> None
+  | Ast0.OptIdent(id) ->
+      call_right left_ident id i (function id -> Ast0.OptIdent(id))
+  | Ast0.UniqueIdent(id) ->
+      call_right left_ident id i (function id -> Ast0.UniqueIdent(id))
 
 let left_fundecl name fninfo =
   let fncall_right processor data cont =
@@ -241,7 +255,16 @@ let left_fundecl name fninfo =
 
 let rec left_decl decl =
   match Ast0.unwrap decl with
-    Ast0.Init(Some stg,ty,id,eq,ini,sem) ->
+    Ast0.MetaDecl(name,pure) ->
+      call_right right_mcode name decl
+	(function name -> Ast0.MetaDecl(name,pure))
+  | Ast0.MetaField(name,pure) ->
+      call_right right_mcode name decl
+	(function name -> Ast0.MetaField(name,pure))
+  | Ast0.MetaFieldList(name,lenname,pure) ->
+      call_right right_mcode name decl
+	(function name -> Ast0.MetaFieldList(name,lenname,pure))
+  | Ast0.Init(Some stg,ty,id,eq,ini,sem) ->
       call_right left_mcode stg decl
 	(function stg -> Ast0.Init(Some stg,ty,id,eq,ini,sem))
   | Ast0.Init(None,ty,id,eq,ini,sem) ->
