@@ -155,38 +155,46 @@ let make_id =
     (function x -> Ast0.OptIdent x)
     (function x -> Ast0.UniqueIdent x)
 
-let ident opt_allowed tgt i =
+let rec ident opt_allowed tgt i =
   match Ast0.unwrap i with
-      Ast0.Id(name) ->
-	let arity =
-	  all_same opt_allowed tgt (mcode2line name)
-	    [mcode2arity name] in
-	let name = mcode name in
-	  make_id i tgt arity (Ast0.Id(name))
-    | Ast0.MetaId(name,constraints,pure) ->
-	let arity =
-	  all_same opt_allowed tgt (mcode2line name)
-	    [mcode2arity name] in
-	let name = mcode name in
-	  make_id i tgt arity (Ast0.MetaId(name,constraints,pure))
-    | Ast0.MetaFunc(name,constraints,pure) ->
-	let arity =
-	  all_same opt_allowed tgt (mcode2line name)
-	    [mcode2arity name] in
-	let name = mcode name in
-	  make_id i tgt arity (Ast0.MetaFunc(name,constraints,pure))
-    | Ast0.MetaLocalFunc(name,constraints,pure) ->
-	let arity =
-	  all_same opt_allowed tgt (mcode2line name)
-	    [mcode2arity name] in
-	let name = mcode name in
-	  make_id i tgt arity (Ast0.MetaLocalFunc(name,constraints,pure))
-    | Ast0.OptIdent(_) | Ast0.UniqueIdent(_) ->
-	failwith "unexpected code"
-
+    Ast0.Id(name) ->
+      let arity =
+	all_same opt_allowed tgt (mcode2line name)
+	  [mcode2arity name] in
+      let name = mcode name in
+      make_id i tgt arity (Ast0.Id(name))
+  | Ast0.MetaId(name,constraints,pure) ->
+      let arity =
+	all_same opt_allowed tgt (mcode2line name)
+	  [mcode2arity name] in
+      let name = mcode name in
+      make_id i tgt arity (Ast0.MetaId(name,constraints,pure))
+  | Ast0.MetaFunc(name,constraints,pure) ->
+      let arity =
+	all_same opt_allowed tgt (mcode2line name)
+	  [mcode2arity name] in
+      let name = mcode name in
+      make_id i tgt arity (Ast0.MetaFunc(name,constraints,pure))
+  | Ast0.MetaLocalFunc(name,constraints,pure) ->
+      let arity =
+	all_same opt_allowed tgt (mcode2line name)
+	  [mcode2arity name] in
+      let name = mcode name in
+      make_id i tgt arity (Ast0.MetaLocalFunc(name,constraints,pure))
+  | Ast0.DisjId(starter,id_list,mids,ender) ->
+      let id_list = List.map (ident opt_allowed tgt) id_list in
+      (match List.rev id_list with
+	_::xs ->
+	  if anyopt xs (function Ast0.OptIdent(_) -> true | _ -> false)
+	  then fail i "opt only allowed in the last disjunct"
+      |	_ -> ());
+      Ast0.rewrap i (Ast0.DisjId(starter,id_list,mids,ender))
+  | Ast0.OptIdent(_) | Ast0.UniqueIdent(_) ->
+      failwith "unexpected code"
+	
 (* --------------------------------------------------------------------- *)
 (* Expression *)
-
+	
 let make_exp =
   make_opt_unique
     (function x -> Ast0.OptExp x)
@@ -411,8 +419,17 @@ and top_typeC tgt opt_allowed typ =
       let arity =
 	all_same opt_allowed tgt (mcode2line kind) [mcode2arity kind] in
       let kind = mcode kind in
-      let name = ident false arity name in
+      let name = get_option (ident false arity) name in
       make_typeC typ tgt arity (Ast0.EnumName(kind,name))
+  | Ast0.EnumDef(ty,lb,decls,rb) ->
+      let arity =
+	all_same opt_allowed tgt (mcode2line lb)
+	  (List.map mcode2arity [lb;rb]) in
+      let ty = typeC arity ty in
+      let lb = mcode lb in
+      let ids = dots (expression tgt) decls in
+      let rb = mcode rb in
+      make_typeC typ tgt arity (Ast0.EnumDef(ty,lb,ids,rb))
   | Ast0.StructUnionName(kind,name) ->
       let arity =
 	all_same opt_allowed tgt (mcode2line kind)
@@ -465,7 +482,19 @@ and make_decl =
 
 and declaration tgt decl =
   match Ast0.unwrap decl with
-    Ast0.Init(stg,ty,id,eq,exp,sem) ->
+    Ast0.MetaDecl(name,pure) ->
+      let arity = all_same true tgt (mcode2line name) [mcode2arity name] in
+      let name = mcode name in
+      make_decl decl tgt arity (Ast0.MetaDecl(name,pure))
+  | Ast0.MetaField(name,pure) ->
+      let arity = all_same true tgt (mcode2line name) [mcode2arity name] in
+      let name = mcode name in
+      make_decl decl tgt arity (Ast0.MetaField(name,pure))
+  | Ast0.MetaFieldList(name,lenname,pure) ->
+      let arity = all_same true tgt (mcode2line name) [mcode2arity name] in
+      let name = mcode name in
+      make_decl decl tgt arity (Ast0.MetaFieldList(name,lenname,pure))
+  | Ast0.Init(stg,ty,id,eq,exp,sem) ->
       let arity =
 	all_same true tgt (mcode2line eq)
 	  ((match stg with None -> [] | Some x -> [mcode2arity x]) @
@@ -545,12 +574,12 @@ and initialiser tgt i =
       make_init i tgt arity (Ast0.MetaInit(name,pure))
   | Ast0.InitExpr(exp) ->
       Ast0.rewrap i (Ast0.InitExpr(expression tgt exp))
-  | Ast0.InitList(lb,initlist,rb) ->
+  | Ast0.InitList(lb,initlist,rb,ordered) ->
       let arity = init_same (mcode2line lb) [mcode2arity lb; mcode2arity rb] in
       let lb = mcode lb in
       let initlist = dots (initialiser arity) initlist in
       let rb = mcode rb in
-      make_init i tgt arity (Ast0.InitList(lb,initlist,rb))
+      make_init i tgt arity (Ast0.InitList(lb,initlist,rb,ordered))
   | Ast0.InitGccExt(designators,eq,ini) ->
       let arity = init_same (mcode2line eq) [mcode2arity eq] in
       let designators = List.map (designator arity) designators in
@@ -946,6 +975,11 @@ and statement tgt stm =
       let inc = mcode inc in
       let s = mcode s in
       make_rule_elem stm tgt arity (Ast0.Include(inc,s))
+  | Ast0.Undef(def,id) ->
+      let arity = all_same true tgt (mcode2line def) [mcode2arity def] in
+      let def = mcode def in
+      let id = ident false arity id in
+      make_rule_elem stm tgt arity (Ast0.Undef(def,id))
   | Ast0.Define(def,id,params,body) ->
       let arity = all_same true tgt (mcode2line def) [mcode2arity def] in
       let def = mcode def in
