@@ -25,7 +25,7 @@
 (* --------------------------------------------------------------------- *)
 (* Modified code *)
 
-type added_string = Noindent of string | Indent of string
+type added_string = Noindent of string | Indent of string | Space of string
 
 type info = { line : int; column : int;
 	      strbef : (added_string * int (* line *) * int (* col *)) list;
@@ -46,6 +46,8 @@ type 'a wrap =
       (* the following is for or expressions *)
       pos_info : meta_name mcode option; (* pos info, try not to duplicate *)
       true_if_test_exp : bool;(* true if "test_exp from iso", only for exprs *)
+      (* the following is only for declarations *)
+      safe_for_multi_decls : bool;
       (* isos relevant to the term; ultimately only used for rule_elems *)
       iso_info : (string*anything) list }
 
@@ -61,7 +63,7 @@ and 'a mcode = 'a * info * mcodekind * meta_pos (* pos variable *)
     (* int list is the match instances, which are only meaningful in annotated
        C code *)
     (* int is the adjacency index, which is incremented on context dots *)
-(* iteration is only allowed on contect code, the intuition vaguely being
+(* iteration is only allowed on context code, the intuition vaguely being
 that there is no way to replace something more than once.  Actually,
 allowing iterated additions on minus code would cause problems with some
 heuristics for adding braces, because one couldn't identify simple
@@ -97,7 +99,8 @@ and end_info =
 and arity = UNIQUE | OPT | MULTI | NONE
 
 and metavar =
-    MetaIdDecl of arity * meta_name (* name *)
+    MetaMetaDecl of arity * meta_name (* name *)
+  | MetaIdDecl of arity * meta_name (* name *)
   | MetaFreshIdDecl of meta_name (* name *) * seed (* seed *)
   | MetaTypeDecl of arity * meta_name (* name *)
   | MetaInitDecl of arity * meta_name (* name *)
@@ -114,6 +117,9 @@ and metavar =
   | MetaLocalIdExpDecl of
       arity * meta_name (* name *) * Type_cocci.typeC list option
   | MetaExpListDecl of arity * meta_name (*name*) * list_len (*len*)
+  | MetaDeclDecl of arity * meta_name (* name *)
+  | MetaFieldDecl of arity * meta_name (* name *)
+  | MetaFieldListDecl of arity * meta_name (* name *) * list_len (*len*)
   | MetaStmDecl of arity * meta_name (* name *)
   | MetaStmListDecl of arity * meta_name (* name *)
   | MetaFuncDecl of arity * meta_name (* name *)
@@ -147,6 +153,7 @@ and base_ident =
   | MetaFunc      of meta_name mcode * idconstraint * keep_binding * inherited
   | MetaLocalFunc of meta_name mcode * idconstraint * keep_binding * inherited
 
+  | DisjId        of ident list
   | OptIdent      of ident
   | UniqueIdent   of ident
 
@@ -275,7 +282,9 @@ and base_typeC =
 
   | Array           of fullType * string mcode (* [ *) *
 	               expression option * string mcode (* ] *)
-  | EnumName        of string mcode (*enum*) * ident (* name *)
+  | EnumName        of string mcode (*enum*) * ident option (* name *)
+  | EnumDef  of fullType (* either EnumName or metavar *) *
+	string mcode (* { *) * expression dots * string mcode (* } *)
   | StructUnionName of structUnion mcode * ident option (* name *)
   | StructUnionDef  of fullType (* either StructUnionName or metavar *) *
 	string mcode (* { *) * declaration dots * string mcode (* } *)
@@ -287,7 +296,7 @@ and fullType = base_fullType wrap
 and typeC = base_typeC wrap
 
 and baseType = VoidType | CharType | ShortType | IntType | DoubleType
-  | FloatType | LongType | LongLongType
+  | FloatType | LongType | LongLongType | SizeType | SSizeType | PtrDiffType
 
 and structUnion = Struct | Union
 
@@ -314,6 +323,8 @@ and base_declaration =
   | Ddots    of string mcode (* ... *) * declaration option (* whencode *)
 
   | MetaDecl of meta_name mcode * keep_binding * inherited
+  | MetaField of meta_name mcode * keep_binding * inherited
+  | MetaFieldList of meta_name mcode * listlen * keep_binding * inherited
 
   | OptDecl    of declaration
   | UniqueDecl of declaration
@@ -326,7 +337,8 @@ and declaration = base_declaration wrap
 and base_initialiser =
     MetaInit of meta_name mcode * keep_binding * inherited
   | InitExpr of expression
-  | InitList of bool (* true if all are - *) *
+  | ArInitList of string mcode (*{*) * initialiser dots * string mcode (*}*)
+  | StrInitList of bool (* true if all are - *) *
         string mcode (*{*) * initialiser list * string mcode (*}*) *
 	initialiser list (* whencode: elements that shouldn't appear in init *)
   | InitGccExt of
@@ -335,6 +347,7 @@ and base_initialiser =
   | InitGccName of ident (* name *) * string mcode (*:*) *
 	initialiser
   | IComma of string mcode (* , *)
+  | Idots  of string mcode (* ... *) * initialiser option (* whencode *)
 
   | OptIni    of initialiser
   | UniqueIni of initialiser
@@ -457,6 +470,7 @@ and base_rule_elem =
   | Ty            of fullType (* only at SP top level, matches a subterm *)
   | TopInit       of initialiser (* only at top level *)
   | Include       of string mcode (*#include*) * inc_file mcode (*file *)
+  | Undef         of string mcode (* #define *) * ident (* name *)
   | DefineHeader  of string mcode (* #define *) * ident (* name *) *
 	             define_parameters (*params*)
   | Case          of string mcode (* case *) * expression * string mcode (*:*)
@@ -575,7 +589,8 @@ and rule =
   | ScriptRule of string (* name *) *
       (* metaname for python (untyped), metavar for ocaml (typed) *)
       string * dependency *
-	(script_meta_name * meta_name * metavar) list * string
+	(script_meta_name * meta_name * metavar) list (*inherited vars*) *
+	meta_name list (*script vars*) * string
   | InitialScriptRule of  string (* name *) *
 	string (*language*) * dependency * string (*code*)
   | FinalScriptRule of  string (* name *) *
@@ -665,8 +680,9 @@ let get_dots_bef_aft x     = x.bef_aft
 let set_dots_bef_aft d x   = {x with bef_aft = d}
 let get_pos x              = x.pos_info
 let set_pos x pos          = {x with pos_info = pos}
-let get_test_exp x      = x.true_if_test_exp
-let set_test_exp x      = {x with true_if_test_exp = true}
+let get_test_exp x         = x.true_if_test_exp
+let set_test_exp x         = {x with true_if_test_exp = true}
+let get_safe_decl x        = x.safe_for_multi_decls
 let get_isos x             = x.iso_info
 let set_isos x isos        = {x with iso_info = isos}
 let get_pos_var (_,_,_,p)  = p
@@ -687,7 +703,8 @@ let get_wcfvs (whencode : ('a wrap, 'b wrap) whencode list) =
 (* --------------------------------------------------------------------- *)
 
 let get_meta_name = function
-    MetaIdDecl(ar,nm) -> nm
+    MetaMetaDecl(ar,nm) -> nm
+  | MetaIdDecl(ar,nm) -> nm
   | MetaFreshIdDecl(nm,seed) -> nm
   | MetaTypeDecl(ar,nm) -> nm
   | MetaInitDecl(ar,nm) -> nm
@@ -700,6 +717,9 @@ let get_meta_name = function
   | MetaIdExpDecl(ar,nm,ty) -> nm
   | MetaLocalIdExpDecl(ar,nm,ty) -> nm
   | MetaExpListDecl(ar,nm,nm1) -> nm
+  | MetaDeclDecl(ar,nm) -> nm
+  | MetaFieldDecl(ar,nm) -> nm
+  | MetaFieldListDecl(ar,nm,nm1) -> nm
   | MetaStmDecl(ar,nm) -> nm
   | MetaStmListDecl(ar,nm) -> nm
   | MetaFuncDecl(ar,nm) -> nm
@@ -759,6 +779,7 @@ let make_term x =
     bef_aft = NoDots;
     pos_info = None;
     true_if_test_exp = false;
+    safe_for_multi_decls = false;
     iso_info = [] }
 
 let make_meta_rule_elem s d (fvs,fresh,inh) =
