@@ -83,36 +83,38 @@ type context = ID | FIELD | FN | GLOBAL
 let is_ifdef name =
   String.length name > 2 && String.uppercase name = name
 
-let ident context old_metas table minus i =
+let rec ident context old_metas table minus i =
   match Ast0.unwrap i with
-      Ast0.Id((name,_,info,_,_,_) : string Ast0.mcode) ->
-	let rl = info.Ast0.pos_info.Ast0.line_start in
-	let is_plus i =
- 	  match Ast0.get_mcodekind i with Ast0.PLUS _ -> true | _ -> false in
-	let err =
-	  if List.exists (function x -> x = name) old_metas
+    Ast0.Id((name,_,info,_,_,_) : string Ast0.mcode) ->
+      let rl = info.Ast0.pos_info.Ast0.line_start in
+      let is_plus i =
+ 	match Ast0.get_mcodekind i with Ast0.PLUS _ -> true | _ -> false in
+      let err =
+	if List.exists (function x -> x = name) old_metas
 	    && (minus || is_plus i)
+	then
+	  begin
+	    warning
+	      (Printf.sprintf
+		 "line %d: %s, previously declared as a metavariable, is used as an identifier" rl name);
+	    true
+	  end
+	else false in
+      (match context with
+	ID ->
+	  if not (is_ifdef name) && minus && not err(* warn only once per id *)
 	  then
-	    begin
-	      warning
-		(Printf.sprintf
-		   "line %d: %s, previously declared as a metavariable, is used as an identifier" rl name);
-	      true
-	    end
-	  else false in
-	  (match context with
-	       ID ->
-		 if not (is_ifdef name) && minus && not err(* warn only once per id *)
-		 then
-		   warning
-		     (Printf.sprintf "line %d: should %s be a metavariable?" rl name)
-	     | _ -> ())
-    | Ast0.MetaId(name,_,_) -> check_table table minus name
-    | Ast0.MetaFunc(name,_,_) -> check_table table minus name
-    | Ast0.MetaLocalFunc(name,_,_) -> check_table table minus name
-    | Ast0.OptIdent(_) | Ast0.UniqueIdent(_) ->
-	failwith "unexpected code"
-
+	    warning
+	      (Printf.sprintf "line %d: should %s be a metavariable?" rl name)
+      | _ -> ())
+  | Ast0.MetaId(name,_,_) -> check_table table minus name
+  | Ast0.MetaFunc(name,_,_) -> check_table table minus name
+  | Ast0.MetaLocalFunc(name,_,_) -> check_table table minus name
+  | Ast0.DisjId(_,id_list,_,_) ->
+      List.iter (ident context old_metas table minus) id_list
+  | Ast0.OptIdent(_) | Ast0.UniqueIdent(_) ->
+      failwith "unexpected code"
+	
 (* --------------------------------------------------------------------- *)
 (* Expression *)
 
@@ -183,6 +185,8 @@ and get_type_name = function
     Type_cocci.ConstVol(_,ty) | Type_cocci.SignedT(_,Some ty)
   | Type_cocci.Pointer(ty)
   | Type_cocci.FunctionPointer(ty) | Type_cocci.Array(ty) -> get_type_name ty
+  | Type_cocci.EnumName(Type_cocci.MV(nm,_,_)) -> Some nm
+  | Type_cocci.StructUnionName(_,Type_cocci.MV(nm,_,_)) -> Some nm
   | Type_cocci.MetaType(nm,_,_) -> Some nm
   | _ -> None
 
@@ -208,7 +212,10 @@ and typeC old_metas table minus t =
       check_table table minus name
   | Ast0.DisjType(_,types,_,_) ->
       List.iter (typeC old_metas table minus) types
-  | Ast0.EnumName(en,id) -> ident GLOBAL old_metas table minus id
+  | Ast0.EnumName(en,Some id) -> ident GLOBAL old_metas table minus id
+  | Ast0.EnumDef(ty,lb,ids,rb) ->
+      typeC old_metas table minus ty;
+      dots (expression GLOBAL old_metas table minus) ids
   | Ast0.StructUnionName(su,Some id) -> ident GLOBAL old_metas table minus id
   | Ast0.StructUnionDef(ty,lb,decls,rb) ->
       typeC old_metas table minus ty;
@@ -224,7 +231,14 @@ and typeC old_metas table minus t =
 
 and declaration context old_metas table minus d =
   match Ast0.unwrap d with
-    Ast0.Init(stg,ty,id,eq,ini,sem) ->
+    Ast0.MetaDecl(name,_) | Ast0.MetaField(name,_) ->
+      check_table table minus name
+  | Ast0.MetaFieldList(name,Ast0.MetaListLen lenname,_) ->
+      check_table table minus name;
+      check_table table minus lenname
+  | Ast0.MetaFieldList(name,_,_) ->
+      check_table table minus name
+  | Ast0.Init(stg,ty,id,eq,ini,sem) ->
       (match Ast0.unwrap ini with
 	Ast0.InitExpr exp ->
 	  typeC old_metas table minus ty;
@@ -263,7 +277,7 @@ and initialiser old_metas table minus ini =
     Ast0.MetaInit(name,_) ->
       check_table table minus name
   | Ast0.InitExpr(exp) -> expression ID old_metas table minus exp
-  | Ast0.InitList(lb,initlist,rb) ->
+  | Ast0.InitList(lb,initlist,rb,ordered) ->
       dots (initialiser old_metas table minus) initlist
   | Ast0.InitGccExt(designators,eq,ini) ->
       List.iter (designator old_metas table minus) designators;
@@ -368,6 +382,8 @@ and statement old_metas table minus s =
       parameter_list old_metas table minus params;
       dots (statement old_metas table minus) body
   | Ast0.Include(inc,s) -> () (* no metavariables possible *)
+  | Ast0.Undef(def,id) ->
+      ident GLOBAL old_metas table minus id
   | Ast0.Define(def,id,params,body) ->
       ident GLOBAL old_metas table minus id;
       define_parameters old_metas table minus params;
@@ -407,6 +423,7 @@ and case_line old_metas table minus c =
     Ast0.Default(def,colon,code) ->
       dots (statement old_metas table minus) code
   | Ast0.Case(case,exp,colon,code) ->
+      expression GLOBAL old_metas table minus exp;
       dots (statement old_metas table minus) code
   | Ast0.DisjCase(_,case_lines,_,_) ->
       List.iter (case_line old_metas table minus) case_lines
