@@ -63,6 +63,8 @@ val all_metadecls : (string, Ast_cocci.metavar list) Hashtbl.t
 
 val clear_meta: (unit -> unit) ref
 
+val add_meta_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
+
 val add_id_meta:
     (Ast_cocci.meta_name -> iconstraints -> Ast0_cocci.pure -> unit) ref
 
@@ -106,6 +108,13 @@ val add_explist_meta:
     (Ast_cocci.meta_name -> Ast_cocci.list_len -> Ast0_cocci.pure ->
       unit) ref
 
+val add_decl_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
+
+val add_field_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
+
+val add_field_list_meta:
+    (Ast_cocci.meta_name -> Ast_cocci.list_len -> Ast0_cocci.pure -> unit) ref
+
 val add_stm_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
 
 val add_stmlist_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
