@@ -130,6 +130,11 @@ let mkddots str (dot,whencode) =
   | ("...",Some [w]) -> Ast0.wrap(Ast0.Ddots(clt2mcode str dot, Some w))
   | _ -> failwith "cannot happen"
 
+let mkddots_one str (dot,whencode) =
+  match str with
+    "..." -> Ast0.wrap(Ast0.Ddots(clt2mcode str dot, whencode))
+  | _ -> failwith "cannot happen"
+
 let mkpdots str dot =
   match str with
     "..." -> Ast0.wrap(Ast0.Pdots(clt2mcode str dot))
@@ -173,6 +178,13 @@ let ty_pointerify ty m =
     (function inner -> function cur -> Type_cocci.Pointer(inner))
     ty m
 
+let arrayify ty ar =
+  List.fold_right
+    (function (l,i,r) ->
+      function rest ->
+	Ast0.wrap (Ast0.Array(rest,clt2mcode "[" l,i,clt2mcode "]" r)))
+    ar ty
+
 (* Left is <=>, Right is =>.  Collect <=>s. *)
 (* The parser should have done this, with precedences.  But whatever... *)
 let iso_adjust first_fn fn first rest =
@@ -200,7 +212,14 @@ let lookup rule name =
 	(Semantic_cocci.Semantic("bad rule "^rule^" or bad variable "^name))
 
 let check_meta_tyopt type_irrelevant = function
-    Ast.MetaIdDecl(Ast.NONE,(rule,name)) ->
+    Ast.MetaMetaDecl(Ast.NONE,(rule,name)) ->
+      (match lookup rule name with
+	Ast.MetaMetaDecl(_,_) -> ()
+      | _ ->
+	  raise
+	    (Semantic_cocci.Semantic
+	       ("incompatible inheritance declaration "^name)))
+  | Ast.MetaIdDecl(Ast.NONE,(rule,name)) ->
       (match lookup rule name with
 	Ast.MetaIdDecl(_,_) | Ast.MetaFreshIdDecl(_,_) -> ()
       | _ ->
@@ -428,7 +447,24 @@ let str2inc s =
   List.map (function "..." -> Ast.IncDots | s -> Ast.IncPath s) elements
 
 (* ---------------------------------------------------------------------- *)
-(* statements *)
+(* declarations and statements *)
+
+let meta_decl name =
+  let (nm,pure,clt) = name in
+  Ast0.wrap(Ast0.MetaDecl(clt2mcode nm clt,pure))
+
+let meta_field name =
+  let (nm,pure,clt) = name in
+  Ast0.wrap(Ast0.MetaField(clt2mcode nm clt,pure))
+
+let meta_field_list name =
+  let (nm,lenname,pure,clt) = name in
+  let lenname =
+    match lenname with
+      Ast.AnyLen -> Ast0.AnyListLen
+    | Ast.MetaLen nm -> Ast0.MetaListLen(clt2mcode nm clt)
+    | Ast.CstLen n -> Ast0.CstListLen n in
+  Ast0.wrap(Ast0.MetaFieldList(clt2mcode nm clt,lenname,pure))
 
 let meta_stm name =
   let (nm,pure,clt) = name in
@@ -502,35 +538,30 @@ let seq lb s rb =
 
 (* ---------------------------------------------------------------------- *)
 
+let check_rule_name = function
+    Some nm ->
+      let n = id2name nm in
+      (try let _ =  Hashtbl.find Data.all_metadecls n in
+      raise (Semantic_cocci.Semantic ("repeated rule name"))
+      with Not_found -> Some n)
+  | None -> None
+
 let make_iso_rule_name_result n =
-    (try let _ =  Hashtbl.find Data.all_metadecls n in
-    raise (Semantic_cocci.Semantic ("repeated rule name"))
-    with Not_found -> ());
-    Ast.CocciRulename (Some n,Ast.NoDep,[],[],Ast.Undetermined,false (*discarded*))
+  (try let _ =  Hashtbl.find Data.all_metadecls n in
+  raise (Semantic_cocci.Semantic ("repeated rule name"))
+  with Not_found -> ());
+  Ast.CocciRulename
+    (Some n,Ast.NoDep,[],[],Ast.Undetermined,false (*discarded*))
 
 let make_cocci_rule_name_result nm d i a e ee =
-  match nm with
-    Some nm ->
-      let n = id2name nm in
-      (try let _ =  Hashtbl.find Data.all_metadecls n in
-      raise (Semantic_cocci.Semantic ("repeated rule name"))
-      with Not_found -> ());
-      Ast.CocciRulename (Some n,d,i,a,e,ee)
-  | None -> Ast.CocciRulename (None,d,i,a,e,ee)
+  Ast.CocciRulename (check_rule_name nm,d,i,a,e,ee)
 
 let make_generated_rule_name_result nm d i a e ee =
-  match nm with
-    Some nm ->
-      let n = id2name nm in
-      (try let _ =  Hashtbl.find Data.all_metadecls n in
-      raise (Semantic_cocci.Semantic ("repeated rule name"))
-      with Not_found -> ());
-      Ast.GeneratedRulename (Some n,d,i,a,e,ee)
-  | None -> Ast.GeneratedRulename (None,d,i,a,e,ee)
+  Ast.GeneratedRulename (check_rule_name nm,d,i,a,e,ee)
 
-let make_script_rule_name_result lang deps =
+let make_script_rule_name_result lang nm deps =
   let l = id2name lang in
-  Ast.ScriptRulename (None,l,deps)
+  Ast.ScriptRulename (check_rule_name nm,l,deps)
 
 let make_initial_script_rule_name_result lang deps =
   let l = id2name lang in
@@ -566,3 +597,30 @@ let verify_parameter_declarations = function
 		   (Ast0.get_line t))
 	  | _ -> ())
 	l
+
+(* ---------------------------------------------------------------------- *)
+(* decide whether an init list is ordered or unordered *)
+
+let struct_initializer initlist =
+  let rec loop i =
+    match Ast0.unwrap i with
+      Ast0.InitGccExt _ -> true
+    | Ast0.InitGccName _ -> true
+    | Ast0.OptIni i | Ast0.UniqueIni i -> loop i
+    | Ast0.MetaInit _ -> true (* ambiguous... *)
+    | _ -> false in
+  let l = Ast0.undots initlist in
+  (l = []) or (List.exists loop l)
+
+let drop_dot_commas initlist =
+  match Ast0.unwrap initlist with
+    Ast0.DOTS(l) ->
+      let rec loop after_comma = function
+	  [] -> []
+	| x::xs ->
+	    (match Ast0.unwrap x with
+	      Ast0.Idots(dots,whencode) -> x :: (loop true xs)
+	    | Ast0.IComma(comma) when after_comma -> (*drop*) loop false xs
+	    | _ -> x :: (loop false xs)) in
+      Ast0.rewrap initlist (Ast0.DOTS(loop false l))
+  | _ -> failwith "not supported"
