@@ -27,6 +27,8 @@ type inherited = bool (* true if inherited *)
 type keep_binding = Unitary (* need no info *)
   | Nonunitary (* need an env entry *) | Saved (* need a witness *)
 
+type meta_name = string * string (*Ast_cocci.meta_name*)
+
 type typeC =
     ConstVol        of const_vol * typeC
   | BaseType        of baseType
@@ -34,16 +36,22 @@ type typeC =
   | Pointer         of typeC
   | FunctionPointer of typeC (* only return type *)
   | Array           of typeC (* drop size info *)
-  | EnumName        of bool (* true if a metaId *) * string
-  | StructUnionName of structUnion * bool (* true if a metaId *) * string
+  | EnumName        of name
+  | StructUnionName of structUnion * name
   | TypeName        of string
-  | MetaType        of (string * string) * keep_binding * inherited
+  | MetaType        of meta_name * keep_binding * inherited
   | Unknown (* for metavariables of type expression *^* *)
 
+and name =
+    NoName
+  | Name of string
+  | MV of meta_name * keep_binding * inherited
+
 and tagged_string = string
 
 and baseType = VoidType | CharType | ShortType | IntType | DoubleType
 | FloatType | LongType | LongLongType | BoolType
+| SizeType | SSizeType | PtrDiffType
 
 and structUnion = Struct | Union
 
@@ -63,8 +71,8 @@ let rec type2c = function
   | Pointer(ty) -> (type2c ty) ^ "*"
   | FunctionPointer(ty) -> (type2c ty) ^ "(*)(...)"
   | Array(ty) -> (type2c ty) ^ "[] "
-  | EnumName(mv,name) -> "enum " ^ name ^ " "
-  | StructUnionName(kind,mv,name) -> (structUnion kind) ^ name ^ " "
+  | EnumName(name) -> "enum " ^ (print_name name)
+  | StructUnionName(kind,name) -> (structUnion kind) ^ (print_name name)
   | TypeName(name) -> name ^ " "
   | MetaType((rule,name),keep,inherited) -> name ^ " "
       (*
@@ -79,6 +87,11 @@ let rec type2c = function
       *)
   | Unknown -> "unknown "
 
+and print_name = function
+    NoName -> ""
+  | MV ((_,name),_,_) -> name ^ " "
+  | Name name -> name ^ " "
+
 and baseType = function
     VoidType -> "void "
   | CharType -> "char "
@@ -89,6 +102,10 @@ and baseType = function
   | LongType -> "long "
   | LongLongType -> "long long "
   | BoolType -> "bool "
+  | SizeType -> "size_t "
+  | SSizeType -> "ssize_t "
+  | PtrDiffType -> "ptrdiff_t "
+
 
 and structUnion = function
     Struct -> "struct "
