@@ -48,6 +48,9 @@ let int_type = T.BaseType(T.IntType)
 let bool_type = T.BaseType(T.BoolType)
 let char_type = T.BaseType(T.CharType)
 let float_type = T.BaseType(T.FloatType)
+let size_type = T.BaseType(T.SizeType)
+let ssize_type = T.BaseType(T.SSizeType)
+let ptrdiff_type = T.BaseType(T.PtrDiffType)
 
 let rec lub_type t1 t2 =
   match (t1,t2) with
@@ -104,6 +107,14 @@ let rec propagate_types env =
     | Ast0.MetaId(id,_,_) ->
 	(try Some(List.assoc (Meta(Ast0.unwrap_mcode id)) env)
 	with Not_found -> None)
+    | Ast0.DisjId(_,id_list,_,_) ->
+	let types = List.map Ast0.get_type id_list in
+	let combined = List.fold_left lub_type None types in
+	(match combined with
+	  None -> None
+	| Some t ->
+	    List.iter (function i -> Ast0.set_type i (Some t)) id_list;
+	    Some t)
     | _ -> k i in
 
   let strip_cv = function
@@ -217,7 +228,7 @@ let rec propagate_types env =
 	| Ast0.RecordAccess(exp,pt,field) ->
 	    (match strip_cv (Ast0.get_type exp) with
 		 None -> None
-	       | Some (T.StructUnionName(_,_,_)) -> None
+	       | Some (T.StructUnionName(_,_)) -> None
 	       | Some (T.TypeName(_)) -> None
 	       | Some (T.MetaType(_,_,_)) -> None
 	       | Some x -> err exp x "non-structure type in field ref")
@@ -229,7 +240,7 @@ let rec propagate_types env =
 		      | Some (T.Unknown) -> None
 		      | Some (T.MetaType(_,_,_)) -> None
 		      | Some (T.TypeName(_)) -> None
-		      | Some (T.StructUnionName(_,_,_)) -> None
+		      | Some (T.StructUnionName(_,_)) -> None
 		      | Some x ->
 			  err exp (T.Pointer(t))
 			    "non-structure pointer type in field ref"
@@ -271,10 +282,11 @@ let rec propagate_types env =
 
   let rec strip id =
     match Ast0.unwrap id with
-      Ast0.Id(name)                -> Id(Ast0.unwrap_mcode name)
-    | Ast0.MetaId(name,_,_)        -> Meta(Ast0.unwrap_mcode name)
-    | Ast0.MetaFunc(name,_,_)      -> Meta(Ast0.unwrap_mcode name)
-    | Ast0.MetaLocalFunc(name,_,_) -> Meta(Ast0.unwrap_mcode name)
+      Ast0.Id(name)                -> [Id(Ast0.unwrap_mcode name)]
+    | Ast0.MetaId(name,_,_)        -> [Meta(Ast0.unwrap_mcode name)]
+    | Ast0.MetaFunc(name,_,_)      -> [Meta(Ast0.unwrap_mcode name)]
+    | Ast0.MetaLocalFunc(name,_,_) -> [Meta(Ast0.unwrap_mcode name)]
+    | Ast0.DisjId(_,id_list,_,_)   -> List.concat (List.map strip id_list)
     | Ast0.OptIdent(id)            -> strip id
     | Ast0.UniqueIdent(id)         -> strip id in
 
@@ -315,12 +327,16 @@ let rec propagate_types env =
 
   and process_decl env decl =
     match Ast0.unwrap decl with
-      Ast0.Init(_,ty,id,_,exp,_) ->
+      Ast0.MetaDecl(_,_) | Ast0.MetaField(_,_)
+    | Ast0.MetaFieldList(_,_,_) -> []
+    | Ast0.Init(_,ty,id,_,exp,_) ->
 	let _ =
 	  (propagate_types env).VT0.combiner_rec_initialiser exp in
-	[(strip id,Ast0.ast0_type_to_type ty)]
+	let ty = Ast0.ast0_type_to_type ty in
+	List.map (function i -> (i,ty)) (strip id)
     | Ast0.UnInit(_,ty,id,_) ->
-	[(strip id,Ast0.ast0_type_to_type ty)]
+	let ty = Ast0.ast0_type_to_type ty in
+	List.map (function i -> (i,ty)) (strip id)
     | Ast0.MacroDecl(_,_,_,_,_) -> []
     | Ast0.TyDecl(ty,_) -> []
               (* pad: should handle typedef one day and add a binding *)
@@ -360,7 +376,8 @@ let rec propagate_types env =
 	let rec get_binding p =
 	  match Ast0.unwrap p with
 	    Ast0.Param(ty,Some id) ->
-	      [(strip id,Ast0.ast0_type_to_type ty)]
+	      let ty = Ast0.ast0_type_to_type ty in
+	      List.map (function i -> (i,ty)) (strip id)
 	  | Ast0.OptParam(param) -> get_binding param
 	  | _ -> [] in
 	let fenv = List.concat (List.map get_binding (Ast0.undots params)) in
