@@ -54,7 +54,8 @@ let meta_pos = function
 (* Modified code *)
 
 let mcodekind brackets fn x info mc =
-  let print = function Ast.Noindent s | Ast.Indent s -> print_string s in
+  let print = function
+      Ast.Noindent s | Ast.Indent s | Ast.Space s -> print_string s in
   List.iter (function (s,_) -> print s) info.Ast0.strings_before;
   (match mc with
     Ast0.MINUS(plus_stream) ->
@@ -122,6 +123,15 @@ let dots between fn d =
       | Ast0.CIRCLES(l) -> print_between between fn l
       | Ast0.STARS(l) -> print_between between fn l)
 
+(* --------------------------------------------------------------------- *)
+(* Disjunctions *)
+
+let do_disj lst processor =
+  print_string "\n("; force_newline();
+  print_between (function _ -> print_string "\n|"; force_newline())
+    processor lst;
+  print_string "\n)"
+
 (* --------------------------------------------------------------------- *)
 
 let print_types = function
@@ -144,6 +154,7 @@ let rec ident i =
       | Ast0.MetaId(name,_,_) -> mcode print_meta name
       | Ast0.MetaFunc(name,_,_) -> mcode print_meta name
       | Ast0.MetaLocalFunc(name,_,_) -> mcode print_meta name
+      | Ast0.DisjId(_,id_list,_,_) -> do_disj id_list ident
       | Ast0.OptIdent(id) -> print_string "?"; ident id
       | Ast0.UniqueIdent(id) -> print_string "!"; ident id)
 
@@ -214,12 +225,7 @@ let rec expression e =
 	  | Ast0.PureContext -> print_string "pure_context")*)
       | Ast0.MetaExprList(name,_,_) -> mcode print_meta name
       | Ast0.EComma(cm) -> mcode print_string cm; print_space()
-      | Ast0.DisjExpr(_,exp_list,_,_) ->
-	  print_string "\n("; force_newline();
-	  print_between
-	    (function _ -> print_string "\n|"; force_newline())
-	    expression exp_list;
-	  print_string "\n)"
+      | Ast0.DisjExpr(_,exp_list,_,_) -> do_disj exp_list expression
       | Ast0.NestExpr(starter,expr_dots,ender,None,multi) ->
 	  mcode print_string starter;
 	  start_block(); dots force_newline expression expr_dots; end_block();
@@ -273,8 +279,13 @@ and typeC t =
       | Ast0.Array(ty,lb,size,rb) ->
 	  typeC ty; mcode print_string lb; print_option expression size;
 	  mcode print_string rb
-      | Ast0.EnumName(kind,name) -> mcode print_string kind; print_string " ";
-	  ident name
+      | Ast0.EnumName(kind,name) ->
+	  mcode print_string kind;
+	  print_option (function x -> ident x; print_string " ") name
+      | Ast0.EnumDef(ty,lb,ids,rb) ->
+	  typeC ty; mcode print_string lb;
+	  dots force_newline expression ids;
+	  mcode print_string rb
       | Ast0.StructUnionName(kind,name) ->
 	  mcode U.structUnion kind;
 	  print_option (function x -> ident x; print_string " ") name
@@ -284,12 +295,7 @@ and typeC t =
 	  mcode print_string rb
       | Ast0.TypeName(name)-> mcode print_string name; print_string " "
       | Ast0.MetaType(name,_)-> mcode print_meta name; print_string " "
-      | Ast0.DisjType(lp,types,mids,rp) ->
-	  print_string "\n"; mcode print_string lp; force_newline();
-	  print_between
-	    (function _ -> print_string "\n|"; force_newline())
-	    typeC types;
-	  print_string "\n"; mcode print_string rp
+      | Ast0.DisjType(_,types,_,_) -> do_disj types typeC
       | Ast0.OptType(ty) -> print_string "?"; typeC ty
       | Ast0.UniqueType(ty) -> print_string "!"; typeC ty)
 
@@ -324,7 +330,10 @@ and declaration d =
   print_context d
     (function _ ->
       match Ast0.unwrap d with
-	Ast0.Init(stg,ty,id,eq,ini,sem) ->
+	Ast0.MetaDecl(name,_) | Ast0.MetaField(name,_)
+      | Ast0.MetaFieldList(name,_,_) ->
+	  mcode print_meta name
+      |	Ast0.Init(stg,ty,id,eq,ini,sem) ->
 	  print_option (mcode U.storage) stg;
 	  print_named_type ty id;
 	  print_string " ";
@@ -365,7 +374,8 @@ and initialiser i =
       match Ast0.unwrap i with
 	Ast0.MetaInit(name,_)-> mcode print_meta name; print_string " "
       |	Ast0.InitExpr(exp) -> expression exp
-      | Ast0.InitList(lb,initlist,rb) ->
+      | Ast0.InitList(lb,initlist,rb,ordered) ->
+          (*doesn't show commas dropped in unordered case*)
 	  mcode print_string lb; open_box 0;
 	  let _ = dots (function _ -> ()) initialiser initlist in
 	  close_box(); mcode print_string rb
@@ -538,6 +548,8 @@ and statement arity s =
 	    whn
       | Ast0.Include(inc,s) ->
 	  mcode print_string inc; print_string " "; mcode U.inc_file s
+      | Ast0.Undef(def,id) ->
+	  mcode print_string def; print_string " "; ident id
       | Ast0.Define(def,id,params,body) ->
 	  mcode print_string def; print_string " "; ident id;
 	  print_define_parameters params;
@@ -640,33 +652,24 @@ let unparse_anything x =
   | Ast0.DotsStmtTag(d) ->
       print_string "StmDots:"; force_newline();
       statement_dots d
-  | Ast0.DotsDeclTag(d) ->
-      declaration_dots d
-  | Ast0.DotsCaseTag(d) ->
-      case_dots d
-  | Ast0.IdentTag(d) ->
-      ident d
+  | Ast0.DotsDeclTag(d) -> declaration_dots d
+  | Ast0.DotsCaseTag(d) -> case_dots d
+  | Ast0.IdentTag(d)    -> ident d
   | Ast0.ExprTag(d) | Ast0.ArgExprTag(d) | Ast0.TestExprTag(d) ->
       print_string "Exp:"; force_newline();
       expression d
-  | Ast0.TypeCTag(d) ->
-      typeC d
-  | Ast0.ParamTag(d) ->
-      parameterTypeDef d
-  | Ast0.InitTag(d) ->
-      initialiser d
-  | Ast0.DeclTag(d) ->
-      declaration d
-  | Ast0.StmtTag(d) ->
+  | Ast0.TypeCTag(d) -> typeC d
+  | Ast0.ParamTag(d) -> parameterTypeDef d
+  | Ast0.InitTag(d)  -> initialiser d
+  | Ast0.DeclTag(d)  -> declaration d
+  | Ast0.StmtTag(d)  ->
       print_string "Stm:"; force_newline();
       statement "" d
-  | Ast0.CaseLineTag(d) ->
-      case_line "" d
-  | Ast0.TopTag(d) ->
-      top_level d
-  | Ast0.IsoWhenTag(x) -> U.print_when_modif x
-  | Ast0.IsoWhenTTag(e) -> expression e
-  | Ast0.IsoWhenFTag(e) -> expression e
+  | Ast0.CaseLineTag(d)  -> case_line "" d
+  | Ast0.TopTag(d)       -> top_level d
+  | Ast0.IsoWhenTag(x)   -> U.print_when_modif x
+  | Ast0.IsoWhenTTag(e)  -> expression e
+  | Ast0.IsoWhenFTag(e)  -> expression e
   | Ast0.MetaPosTag(var) -> meta_pos var);
   quiet := q;
   print_newline()
